@@ -123,7 +123,8 @@ void QSvgPath::drawCommand(QPainter *p, QSvgExtraStates &states)
         p->drawPoint(m_path.boundingRect().topLeft());
     else
         p->drawPath(m_path);
-    QSvgMarker::drawMarkersForNode(this, p, states);
+    if (!path().isEmpty())
+        QSvgMarker::drawMarkersForNode(this, p, states);
 }
 
 bool QSvgPath::separateFillStroke() const
@@ -368,13 +369,15 @@ bool QSvgText::shouldDrawNode(QPainter *p, QSvgExtraStates &) const
     return true;
 }
 
+bool QSvgText::separateFillStroke() const
+{
+    return true;
+}
+
 void QSvgText::draw_helper(QPainter *p, QSvgExtraStates &states, QRectF *boundingRect) const
 {
     const bool isPainting = (boundingRect == nullptr);
     if (!isPainting || shouldDrawNode(p, states)) {
-        qreal oldOpacity = p->opacity();
-        p->setOpacity(oldOpacity * states.fillOpacity);
-
         QFont font = p->font();
         Qt::Alignment alignment = states.textAnchor;
 
@@ -446,8 +449,11 @@ void QSvgText::draw_helper(QPainter *p, QSvgExtraStates &states, QRectF *boundin
                 range.start = paragraphs.back().size();
                 range.length = newText.size();
                 range.format.setFont(font);
-                range.format.setTextOutline(p->pen());
-                range.format.setForeground(p->brush());
+                if (p->pen().style() != Qt::NoPen && p->pen().brush() != Qt::NoBrush)
+                    range.format.setTextOutline(p->pen());
+                // work around QTBUG-136696: NoBrush fills the foreground with the outline's pen
+                range.format.setForeground(p->brush().style() == Qt::NoBrush
+                                           ? QColor(Qt::transparent) : p->brush());
 
                 if (appendSpace) {
                     Q_ASSERT(!formatRanges.back().isEmpty());
@@ -541,12 +547,10 @@ void QSvgText::draw_helper(QPainter *p, QSvgExtraStates &states, QRectF *boundin
                 *boundingRect = brect;
             }
         }
-
-        p->setOpacity(oldOpacity);
     }
 }
 
-void QSvgText::addText(const QString &text)
+void QSvgText::addText(QStringView text)
 {
     m_tspans.append(new QSvgTspan(this, false));
     m_tspans.back()->setWhitespaceMode(m_mode);
